@@ -27,8 +27,8 @@ use pact_matching::*;
 use pact_matching::models::*;
 use pact_matching::models::generators::GeneratorTestMode;
 use pact_matching::models::http_utils::HttpAuth;
-use pact_matching::models::json_utils::json_to_string;
 use pact_matching::models::v4::V4Interaction;
+use pact_models::json_utils::json_to_string;
 use pact_models::provider_states::*;
 
 use crate::callback_executors::{ProviderStateError, ProviderStateExecutor};
@@ -561,17 +561,12 @@ pub async fn verify_provider_async<F: RequestFilterExecutor, S: ProviderStateExe
 ) -> bool {
     let pact_results = fetch_pacts(source, consumers).await;
 
-    let mut results: Vec<(Option<String>, Option<MismatchResult>)> = vec![];
+    let mut results: Vec<(Option<String>, Result<(), MismatchResult>)> = vec![];
     let mut pending_errors: Vec<(String, MismatchResult)> = vec![];
-    let mut all_errors: Vec<(String, MismatchResult)> = vec![];
+    let mut errors: Vec<(String, MismatchResult)> = vec![];
     for pact_result in pact_results {
       match pact_result {
         Ok((pact, context, pact_source)) => {
-          let pending = match &context {
-            Some(context) => context.verification_properties.pending,
-            None => false
-          };
-
           display_notices(&context, VERIFICATION_NOTICE_BEFORE);
           println!("\nVerifying a pact between {} and {}",
           Style::new().bold().paint(pact.consumer().name.clone()),
@@ -580,15 +575,19 @@ pub async fn verify_provider_async<F: RequestFilterExecutor, S: ProviderStateExe
           if pact.interactions().is_empty() {
             println!("         {}", Yellow.paint("WARNING: Pact file has no interactions"));
           } else {
-            let errors = verify_pact(&provider_info, &filter, pact, &options,
-                                     &provider_state_executor.clone()).await;
-            for (id, desc, error) in &errors {
-              results.push((id.clone(), error.clone()));
-              if let Some(error) = error {
-                if pending {
-                  pending_errors.push((desc.clone(), error.clone()));
+            let pending = match &context {
+              Some(context) => context.verification_properties.pending,
+              None => false
+            };
+            let result = verify_pact_internal(&provider_info, &filter, pact, &options,
+                                              &provider_state_executor.clone(), pending).await;
+            for result in &result.results {
+              results.push((result.interaction_id.clone(), result.result.clone()));
+              if let Err(error) = &result.result {
+                if result.pending {
+                  pending_errors.push((result.description.clone(), error.clone()));
                 } else {
-                  all_errors.push((desc.clone(), error.clone()));
+                  errors.push((result.description.clone(), error.clone()));
                 }
               }
             }
@@ -596,13 +595,13 @@ pub async fn verify_provider_async<F: RequestFilterExecutor, S: ProviderStateExe
             if options.publish {
               publish_result(&results, &pact_source, &options).await;
 
-              if !all_errors.is_empty() || !pending_errors.is_empty() {
+              if !errors.is_empty() || !pending_errors.is_empty() {
                 display_notices(&context, VERIFICATION_NOTICE_AFTER_ERROR_RESULT_AND_PUBLISH);
               } else {
                 display_notices(&context, VERIFICATION_NOTICE_AFTER_SUCCESSFUL_RESULT_AND_PUBLISH);
               }
             } else {
-              if !all_errors.is_empty() || pending_errors.is_empty() {
+              if !errors.is_empty() || pending_errors.is_empty() {
                 display_notices(&context, VERIFICATION_NOTICE_AFTER_ERROR_RESULT_AND_NO_PUBLISH);
               } else {
                 display_notices(&context, VERIFICATION_NOTICE_AFTER_SUCCESSFUL_RESULT_AND_NO_PUBLISH);
@@ -611,8 +610,8 @@ pub async fn verify_provider_async<F: RequestFilterExecutor, S: ProviderStateExe
           }
         },
         Err(err) => {
-          log::error!("Failed to load pact - {}", Red.paint(err.to_string()));
-          all_errors.push(("Failed to load pact".to_string(), MismatchResult::Error(err.to_string(), None)));
+          error!("Failed to load pact - {}", Red.paint(err.to_string()));
+          errors.push(("Failed to load pact".to_string(), MismatchResult::Error(err.to_string(), None)));
         }
       }
     };
@@ -620,12 +619,12 @@ pub async fn verify_provider_async<F: RequestFilterExecutor, S: ProviderStateExe
     if !pending_errors.is_empty() {
       println!("\nPending Failures:\n");
       print_errors(&pending_errors);
-      println!("\nThere were {} non-fatal pact failures on pending pacts (see docs.pact.io/pending for more)\n", pending_errors.len());
+      println!("\nThere were {} non-fatal pact failures on pending pacts or interactions (see docs.pact.io/pending for more information)\n", pending_errors.len());
     }
-    if !all_errors.is_empty() {
+    if !errors.is_empty() {
       println!("\nFailures:\n");
-      print_errors(&all_errors);
-      println!("\nThere were {} pact failures\n", all_errors.len());
+      print_errors(&errors);
+      println!("\nThere were {} pact failures\n", errors.len());
       false
     } else {
       println!();
@@ -756,58 +755,99 @@ async fn fetch_pacts(source: Vec<PactSource>, consumers: Vec<String>)
     .await
 }
 
+/// /// Result of verifying a Pact interaction
+pub struct VerificationInteractionResult {
+  /// Interaction ID
+  pub interaction_id: Option<String>,
+  /// Description
+  pub description: String,
+  /// Result of the verification
+  pub result: Result<(), MismatchResult>,
+  /// If the Pact or interaction is pending
+  pub pending: bool
+}
+
+/// Result of verifying a Pact
+pub struct VerificationResult {
+  /// Results that occurred
+  pub results: Vec<VerificationInteractionResult>
+}
+
 /// Internal function, public for testing purposes
-pub async fn verify_pact<'a, F: RequestFilterExecutor, S: ProviderStateExecutor>(
+pub async fn verify_pact_internal<'a, F: RequestFilterExecutor, S: ProviderStateExecutor>(
   provider_info: &ProviderInfo,
   filter: &FilterInfo,
   pact: Box<dyn Pact + 'a>,
   options: &VerificationOptions<F>,
-  provider_state_executor: &Arc<S>
-) -> Vec<(Option<String>, String, Option<MismatchResult>)> {
-    let mut errors: Vec<(Option<String>, String, Option<MismatchResult>)> = vec![];
+  provider_state_executor: &Arc<S>,
+  pending: bool
+) -> VerificationResult {
+  let results: Vec<(&dyn Interaction, Result<Option<String>, MismatchResult>)> = futures::stream::iter(
+    pact.interactions().iter().cloned()
+  )
+    .filter(|interaction| futures::future::ready(filter_interaction(*interaction, filter)))
+    .then( |interaction| async move {
+      verify_interaction(provider_info, interaction, options, provider_state_executor)
+        .then(|result| futures::future::ready((interaction, result)))
+        .await
+    })
+    .collect()
+    .await;
 
-    let results: Vec<(&dyn Interaction, Result<Option<String>, MismatchResult>)> = futures::stream::iter(
-      pact.interactions().iter().cloned()
-    )
-      .filter(|interaction| futures::future::ready(filter_interaction(*interaction, filter)))
-      .then( |interaction| async move {
-        verify_interaction(provider_info, interaction, options, provider_state_executor)
-          .then(|result| futures::future::ready((interaction, result)))
-          .await
-      })
-      .collect()
-      .await;
-
-    for (interaction, match_result) in results {
-      let mut description = format!("Verifying a pact between {} and {}",
-                                    pact.consumer().name.clone(), pact.provider().name.clone());
-      if let Some((first, elements)) = interaction.provider_states().split_first() {
-        description.push_str(&format!(" Given {}", first.name));
-        for state in elements {
-          description.push_str(&format!(" And {}", state.name));
-        }
+  let mut errors: Vec<VerificationInteractionResult> = vec![];
+  for (interaction, match_result) in results {
+    let mut description = format!("Verifying a pact between {} and {}",
+      pact.consumer().name.clone(), pact.provider().name.clone());
+    if let Some((first, elements)) = interaction.provider_states().split_first() {
+      description.push_str(&format!(" Given {}", first.name));
+      for state in elements {
+        description.push_str(&format!(" And {}", state.name));
       }
-      description.push_str(" - ");
-      description.push_str(&interaction.description());
+    }
+    description.push_str(" - ");
+    description.push_str(&interaction.description());
+    if interaction.pending() {
+      println!("  {} {}", interaction.description(), Yellow.paint("[PENDING]"));
+    } else {
       println!("  {}", interaction.description());
+    };
 
-      if interaction.is_v4() {
-        if let Some(interaction) = interaction.as_v4() {
-          display_comments(interaction)
-        }
-      }
-
-      if let Some(interaction) = interaction.as_request_response() {
-        display_request_response_result(&mut errors, &interaction, &match_result, &description)
-      }
-      if let Some(interaction) = interaction.as_message() {
-        display_message_result(&mut errors, &interaction, &match_result, &description)
+    if interaction.is_v4() {
+      if let Some(interaction) = interaction.as_v4() {
+        display_comments(interaction)
       }
     }
 
-    println!();
+    if let Some(interaction) = interaction.as_request_response() {
+      display_request_response_result(&interaction, &match_result)
+    }
+    if let Some(interaction) = interaction.as_message() {
+      display_message_result(&interaction, &match_result)
+    }
 
-    errors
+    match match_result {
+      Ok(_) => {
+        errors.push(VerificationInteractionResult {
+          interaction_id: interaction.id(),
+          description: description.clone(),
+          result: Ok(()),
+          pending: pending || interaction.pending()
+        });
+      },
+      Err(err) => {
+        errors.push(VerificationInteractionResult {
+          interaction_id: interaction.id(),
+          description: description.clone(),
+          result: Err(err.clone()),
+          pending: pending || interaction.pending()
+        });
+      }
+    }
+  }
+
+  println!();
+
+  VerificationResult { results: errors }
 }
 
 fn display_comments(interaction: Box<dyn V4Interaction>) {
@@ -830,18 +870,22 @@ fn display_comments(interaction: Box<dyn V4Interaction>) {
 }
 
 async fn publish_result<F: RequestFilterExecutor>(
-  results: &[(Option<String>, Option<MismatchResult>)],
+  results: &[(Option<String>, Result<(), MismatchResult>)],
   source: &PactSource,
   options: &VerificationOptions<F>
 ) {
   if let PactSource::BrokerUrl(_, broker_url, auth, links) = source.clone() {
-    log::info!("Publishing verification results back to the Pact Broker");
-    let result = if results.iter().all(|(_, result)| result.is_none()) {
-      log::debug!("Publishing a successful result to {}", source);
+    info!("Publishing verification results back to the Pact Broker");
+    let result = if results.iter().all(|(_, result)| result.is_ok()) {
+      debug!("Publishing a successful result to {}", source);
       TestResult::Ok(results.iter().map(|(id, _)| id.clone()).collect())
     } else {
-      log::debug!("Publishing a failure result to {}", source);
-      TestResult::Failed(Vec::from(results))
+      debug!("Publishing a failure result to {}", source);
+      TestResult::Failed(
+        results.iter()
+        .map(|(id, result)| (id.clone(), result.as_ref().err().cloned()))
+        .collect()
+      )
     };
     let provider_version = options.provider_version.clone().unwrap();
     let publish_result = publish_verification_results(
@@ -854,9 +898,9 @@ async fn publish_result<F: RequestFilterExecutor>(
       options.provider_tags.clone()
     ).await;
 
-    match publish_result {
-      Ok(_) => log::info!("Results published to Pact Broker"),
-      Err(ref err) => log::error!("Publishing of verification results failed with an error: {}", err)
+    match &publish_result {
+      Ok(_) => info!("Results published to Pact Broker"),
+      Err(err) => error!("Publishing of verification results failed with an error: {}", err)
     };
   }
 }
